﻿using System;
using System.Threading;
using System.Threading.Tasks;

namespace WaitVsAwait
{
    class AsyncClass
    {
        public void Process(string s)
        {
            if (s == null) throw new ArgumentNullException("s");

            ProcessCore(s);
        }

        public Task ProcessAsync1(string s)
        {
            if (s == null) throw new ArgumentNullException("s");

            return Task.Run(() => ProcessCore(s));
        }

        public async Task ProcessAsync2(string s)
        {
            if (s == null) throw new ArgumentNullException("s");

            await Task.Run(() => ProcessCore(s));
        }

        private void ProcessCore(string s)
        {
            for (int len = 1; len <= s.Length; len++)
            {
                Thread.Sleep(1000); // This task is very complicated!!
                Console.WriteLine(s.Substring(0, len));
            }
        }
    }

    class Program
    {
        static void Main()
        {
            DoWork1();
            //DoWork2().Wait();
        }

        static void DoWork1()
        {
            var asyncTest = new AsyncClass();
            try
            {
                asyncTest.Process(null);                                 // ArgumentNullException
                //asyncTest.ProcessAsync1(null);                           // ArgumentNullException
                //asyncTest.ProcessAsync1(null).Wait();                    // ArgumentNullException  
                //asyncTest.ProcessAsync1(null).GetAwaiter().GetResult();  // ArgumentNullException
                //asyncTest.ProcessAsync2(null);                           // No Exception!
                //asyncTest.ProcessAsync2(null).Wait();                    // AggregateException  
                //asyncTest.ProcessAsync2(null).GetAwaiter().GetResult();  // ArgumentNullException
            }
            catch (ArgumentNullException ex)
            {
                Console.WriteLine("Argument null exception thrown.");
                Console.WriteLine(ex);
            }
            catch (AggregateException ex)
            {
                Console.WriteLine("Aggregate exception thrown");
                Console.WriteLine(ex);
            }
        }

        static async Task DoWork2()
        {
            var asyncTest = new AsyncClass();
            try
            {
                await asyncTest.ProcessAsync1(null);    // ArgumentNullException
                //await asyncTest.ProcessAsync2(null);    // ArgumentNullException
            }
            catch (ArgumentNullException ex)
            {
                Console.WriteLine("Argument null exception thrown.");
                Console.WriteLine(ex);
            }
            catch (AggregateException ex)
            {
                Console.WriteLine("Aggregate exception thrown");
                Console.WriteLine(ex);
            }
        }
    }
}
